#include "/home/vpx500/BSP/ltib-imx21ads-20070609/rpm/BUILD/celinux-040503/include/linux/soundcard.h"
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/ioctl.h>

const char *labels[] = SOUND_DEVICE_LABELS;

int fd, sndfd, result;
int ioctl_req;
u_int32_t nreg[2];
u_int8_t breg[4];

struct audio_buf_info bufinfo;

void print_devices(u_int32_t bitmask)
{
	u_int32_t testbit, cnt;
	cnt = 0;

	printf("available mixer devices:\r\n");
	if(bitmask == 0)
	{
		printf("no devices available\r\n");
	}
	else
	{
		for(testbit = 0x01; testbit != (1<<SOUND_MIXER_NRDEVICES) ; testbit <<= 1)
		{
			if(testbit & bitmask)
			{
				ioctl_req = MIXER_READ(cnt);
				result = ioctl(fd, ioctl_req, breg); //reg);
				printf("%s (0x%08X) - level at %i %i %i %i\r\n", labels[cnt], ioctl_req, breg[0],breg[1],breg[2],breg[3]);
			}
			cnt++;
		}
	}
	cnt = SOUND_MIXER_SPEAKER;
	ioctl_req = MIXER_READ(cnt);
	result = ioctl(fd, ioctl_req, breg); //reg);
	printf("%s (0x%08X) - level at %i %i %i %i\r\n", labels[cnt], ioctl_req, breg[0],breg[1],breg[2],breg[3]);
}

void print_dspformats(u_int32_t fmts)
{
	printf("supported formats:\r\n");
	if(fmts & AFMT_MU_LAW)
		printf("µLaw\r\n");
	if(fmts & AFMT_A_LAW)
		printf("aLaw\r\n");
	if(fmts & AFMT_IMA_ADPCM)
		printf("IMA ADPCM\r\n");
	if(fmts & AFMT_U8)
		printf("U8\r\n");
	if(fmts & AFMT_S16_LE)
		printf("S16 LE\r\n");
	if(fmts & AFMT_S16_BE)
		printf("S16 BE\r\n");
	if(fmts & AFMT_S8)
		printf("S8\r\n");
	if(fmts & AFMT_U16_LE)
		printf("U16 LE\r\n");
	if(fmts & AFMT_U16_BE)
		printf("U16 BE\r\n");
	if(fmts & AFMT_MPEG)
		printf("MPEG\r\n");
	if(fmts & AFMT_AC3)
		printf("AC3\r\n");

}

void print_bufinfo(void)
{
	printf("fragments available: %i\r\n", bufinfo.fragments);
	printf("fragments allocated: %i\r\n", bufinfo.fragstotal);
	printf("fragment size:       %i\r\n", bufinfo.fragsize);
	printf("space available:     %i\r\n", bufinfo.bytes);
}

void print_dspcaps(u_int32_t caps)
{
	printf("dsp capabilities:\r\n");
	printf("revision: 0x%02X\r\n", caps & DSP_CAP_REVISION);
	if(caps & DSP_CAP_DUPLEX)
		printf("Full Duplex\r\n");
	if(caps & DSP_CAP_REALTIME)
		printf("Realtime\r\n");
	if(caps & DSP_CAP_BATCH)
		printf("Batch\r\n");
	if(caps & DSP_CAP_COPROC)
		printf("CoProcessor\r\n");
	if(caps & DSP_CAP_TRIGGER)
		printf("Trigger\r\n");
	if(caps & DSP_CAP_MMAP)
		printf("mmap\r\n");
	if(caps & DSP_CAP_MULTI)
		printf("Multi\r\n");
	if(caps & DSP_CAP_BIND)
		printf("Bind\r\n");

	
}

int main(void)
{
	float x,y,z;

	x = 0.234;
	y = 1.7245;

	fd = open("/dev/mixer1", O_RDWR);
	if(fd == -1)
	{
		printf("can't open /dev/mixer1\r\n");
		goto EXIT;
	}

	sndfd = open("/dev/dsp1", O_WRONLY);
	if(sndfd == -1)
	{
		printf("can't open /dev/dsp1\r\n");
		goto EXIT;
	}

	nreg[0] = 0x0000;
	nreg[1] = 0x0000;

	ioctl_req = SOUND_MIXER_READ_DEVMASK;
	result = ioctl(fd, ioctl_req, nreg); //reg);

	printf("0x%08X (0x%08X) = %08X,%08X\r\n", ioctl_req,result,nreg[0],nreg[1]);
	print_devices(nreg[0]);

	ioctl_req = SOUND_MIXER_READ_STEREODEVS;
	result = ioctl(fd, ioctl_req, nreg); //reg);

	printf("0x%08X (0x%08X) = %08X,%08X\r\n", ioctl_req,result,nreg[0],nreg[1]);

	ioctl_req = SOUND_MIXER_READ_RECMASK;
	result = ioctl(fd, ioctl_req, nreg); //reg);

	printf("0x%08X (0x%08X) = %08X,%08X\r\n", ioctl_req,result,nreg[0],nreg[1]);

	nreg[0] = 0;
	nreg[1] = 0;
	ioctl_req = SNDCTL_DSP_GETFMTS;
	result = ioctl(sndfd, ioctl_req, nreg); //reg);

	printf("0x%08X (0x%08X) = %08X,%08X\r\n", ioctl_req,result,nreg[0],nreg[1]);
	print_dspformats(nreg[0]);

	ioctl_req = SNDCTL_DSP_SETFMT;
	nreg[0] = 0;
	nreg[1] = 0;
	result = ioctl(sndfd, ioctl_req, nreg); //reg);
	printf("0x%08X (0x%08X) = %08X,%08X\r\n", ioctl_req,result,nreg[0],nreg[1]);
	print_dspformats(nreg[0]);

	ioctl_req = SNDCTL_DSP_GETOSPACE;
	result = ioctl(sndfd, ioctl_req, &bufinfo); //reg);
	printf("GETOSPACE: (0x%08X - 0x%08X)\r\n",ioctl_req,result);
	print_bufinfo();

	ioctl_req = SNDCTL_DSP_GETISPACE;
	result = ioctl(sndfd, ioctl_req, &bufinfo); //reg);
	printf("GETISPACE: (0x%08X - 0x%08X)\r\n",ioctl_req,result);
	print_bufinfo();

	ioctl_req = SNDCTL_DSP_GETCAPS;
	nreg[0] = 0;
	nreg[1] = 0;
	result = ioctl(sndfd, ioctl_req, nreg); //reg);
	printf("0x%08X (0x%08X) = %08X,%08X\r\n", ioctl_req,result,nreg[0],nreg[1]);
	print_dspcaps(nreg[0]);

	printf("SOUND_PCM_READ_RATE       = 0x%08X\r\n", SOUND_PCM_READ_RATE);
	printf("SOUND_PCM_READ_CHANNELS   = 0x%08X\r\n", SOUND_PCM_READ_CHANNELS);
	printf("SOUND_PCM_READ_BITS       = 0x%08X\r\n", SOUND_PCM_READ_BITS);
	printf("SOUND_PCM_READ_FILTER     = 0x%08X\r\n", SOUND_PCM_READ_FILTER);
	printf("SOUND_PCM_WRITE_BITS      = 0x%08X\r\n", SOUND_PCM_WRITE_BITS);
	printf("SOUND_PCM_WRITE_RATE      = 0x%08X\r\n", SOUND_PCM_WRITE_RATE);
	printf("SOUND_PCM_POST            = 0x%08X\r\n", SOUND_PCM_POST);
	printf("SOUND_PCM_RESET           = 0x%08X\r\n", SOUND_PCM_RESET);
	printf("SOUND_PCM_SYNC            = 0x%08X\r\n", SOUND_PCM_SYNC);
	printf("SOUND_PCM_SUBDIVIDE       = 0x%08X\r\n", SOUND_PCM_SUBDIVIDE);
	printf("SOUND_PCM_SETFRAGMENT     = 0x%08X\r\n", SOUND_PCM_SETFRAGMENT);
	printf("SOUND_PCM_GETFMTS         = 0x%08X\r\n", SOUND_PCM_GETFMTS);
	printf("SOUND_PCM_SETFMT          = 0x%08X\r\n", SOUND_PCM_SETFMT);
	printf("SOUND_PCM_GETOSPACE       = 0x%08X\r\n", SOUND_PCM_GETOSPACE);
	printf("SOUND_PCM_GETISPACE       = 0x%08X\r\n", SOUND_PCM_GETISPACE);
	printf("SOUND_PCM_NONBLOCK        = 0x%08X\r\n", SOUND_PCM_NONBLOCK);
	printf("SOUND_PCM_GETCAPS         = 0x%08X\r\n", SOUND_PCM_GETCAPS);
	printf("SOUND_PCM_GETTRIGGER      = 0x%08X\r\n", SOUND_PCM_GETTRIGGER);
	printf("SOUND_PCM_SETTRIGGER      = 0x%08X\r\n", SOUND_PCM_SETTRIGGER);
	printf("SOUND_PCM_SETSYNCRO       = 0x%08X\r\n", SOUND_PCM_SETSYNCRO);
	printf("SOUND_PCM_GETIPTR         = 0x%08X\r\n", SOUND_PCM_GETIPTR);
	printf("SOUND_PCM_GETOPTR         = 0x%08X\r\n", SOUND_PCM_GETOPTR);
	printf("SOUND_PCM_MAPINBUF        = 0x%08X\r\n", SOUND_PCM_MAPINBUF);
	printf("SOUND_PCM_MAPOUTBUF       = 0x%08X\r\n", SOUND_PCM_MAPOUTBUF);

	printf("SOUND_PCM_WRITE_CHANNELS  = 0x%08X\r\n", SOUND_PCM_WRITE_CHANNELS);
	printf("SOUND_PCM_WRITE_FILTER    = 0x%08X\r\n", SOUND_PCM_WRITE_FILTER);

	printf("SNDCTL_DSP_RESET          = 0x%08X\r\n", SNDCTL_DSP_RESET);
	printf("SNDCTL_DSP_SYNC           = 0x%08X\r\n", SNDCTL_DSP_SYNC);
	printf("SNDCTL_DSP_SPEED          = 0x%08X\r\n", SNDCTL_DSP_SPEED);
	printf("SNDCTL_DSP_STEREO         = 0x%08X\r\n", SNDCTL_DSP_STEREO);
	printf("SNDCTL_DSP_GETBLKSIZE     = 0x%08X\r\n", SNDCTL_DSP_GETBLKSIZE);
	printf("SNDCTL_DSP_SAMPLESIZE     = 0x%08X\r\n", SNDCTL_DSP_SAMPLESIZE);
	printf("SNDCTL_DSP_CHANNELS       = 0x%08X\r\n", SNDCTL_DSP_CHANNELS);
	printf("SNDCTL_DSP_POST           = 0x%08X\r\n", SNDCTL_DSP_POST);
	printf("SNDCTL_DSP_SUBDIVIDE      = 0x%08X\r\n", SNDCTL_DSP_SUBDIVIDE);
	printf("SNDCTL_DSP_SETFRAGMENT    = 0x%08X\r\n", SNDCTL_DSP_SETFRAGMENT);
	printf("SNDCTL_DSP_GETFMTS        = 0x%08X\r\n", SNDCTL_DSP_GETFMTS);
	printf("SNDCTL_DSP_SETFMT         = 0x%08X\r\n", SNDCTL_DSP_SETFMT);
	printf("SNDCTL_DSP_GETOSPACE      = 0x%08X\r\n", SNDCTL_DSP_GETOSPACE);
	printf("SNDCTL_DSP_GETISPACE      = 0x%08X\r\n", SNDCTL_DSP_GETISPACE);
	printf("SNDCTL_DSP_NONBLOCK       = 0x%08X\r\n", SNDCTL_DSP_NONBLOCK);
	printf("SNDCTL_DSP_GETCAPS        = 0x%08X\r\n", SNDCTL_DSP_GETCAPS);
	printf("SNDCTL_DSP_GETTRIGGER     = 0x%08X\r\n", SNDCTL_DSP_GETTRIGGER);
	printf("SNDCTL_DSP_SETTRIGGER     = 0x%08X\r\n", SNDCTL_DSP_SETTRIGGER);
	printf("SNDCTL_DSP_GETIPTR        = 0x%08X\r\n", SNDCTL_DSP_GETIPTR);
	printf("SNDCTL_DSP_GETOPTR        = 0x%08X\r\n", SNDCTL_DSP_GETOPTR);
	printf("SNDCTL_DSP_MAPINBUF       = 0x%08X\r\n", SNDCTL_DSP_MAPINBUF);
	printf("SNDCTL_DSP_MAPOUTBUF      = 0x%08X\r\n", SNDCTL_DSP_MAPOUTBUF);
	printf("SNDCTL_DSP_SETSYNCRO      = 0x%08X\r\n", SNDCTL_DSP_SETSYNCRO);
	printf("SNDCTL_DSP_SETDUPLEX      = 0x%08X\r\n", SNDCTL_DSP_SETDUPLEX);
	printf("SNDCTL_DSP_GETODELAY      = 0x%08X\r\n", SNDCTL_DSP_GETODELAY);
	printf("SNDCTL_DSP_GETCHANNELMASK = 0x%08X\r\n", SNDCTL_DSP_GETCHANNELMASK);
	printf("SNDCTL_DSP_BIND_CHANNEL   = 0x%08X\r\n", SNDCTL_DSP_BIND_CHANNEL);
	printf("SNDCTL_DSP_SETSPDIF       = 0x%08X\r\n", SNDCTL_DSP_SETSPDIF);
	printf("SNDCTL_DSP_GETSPDIF       = 0x%08X\r\n", SNDCTL_DSP_GETSPDIF);
	printf("SNDCTL_DSP_PROFILE        = 0x%08X\r\n", SNDCTL_DSP_PROFILE);

	z = x*y;
	printf("floats: %f %f %f\r\n", x,y,z);

EXIT:
	if(close(fd) == -1)
	{
		printf("can't close /dev/mixer1\r\n");
	}

	if(close(sndfd) == -1)
	{
		printf("can't close /dev/dsp1\r\n");
	}

	return 0;
}
